/* USER CODE BEGIN Header */
/* F446RE_oled_RTC
 *
 * Csoportos adatbeolvasás egy DS32231 RTC-ről.
 * A beolvasott adatok az RTC-ből kiolvasott
 * aktuális időt és dátumot adják meg.
 * Az alábbi programban az időt egy SSD1306
 * 128x32 felbontású OLED kijelzőn jelenítjük meg.
 * Az OLED kijelző is az I2C buszra csatlakozik
 *
 * Az I2C kapcsolathoz az I2C1 csatornát és a
 * PB8 (SCL), PB9 (SDA) kivezetéseket használjuk.
 * A kommunikáció standard módban (100 kHz), 1:1
 * kitöltéssel (F/S = 0, DUTY = 0) megy.
 *
 * Kapcsolás:
 *   --------        -------------        ------------------------
 *  | STM32  |      | DS3231 RTC  |      | SSD1306 128x32         |
 *  |    PB9 |------| SDA         |------| SDA  2021-FEB-18 THU   |
 *  |    PB8 |------| SCL         |------| SCL      _     _   _   |
 *  |     5V |------| VCC         |------| VCC   |   | * | | | |  |
 *  |    GND |------| GND         |------| GND   |   | * |_| |_|  |
 *   --------        -------------        ------------------------
 *
 * Felhasznált források:
 *
 * Controllerstech: OLED display using I2C and STM32
 * https://controllerstech.com/oled-display-using-i2c-stm32/
 *
 * Alexander Lutsai: Блиблиотека OLED ди�?пле�? SSD1306 дл�? STM32 микроконтроллеров
 * https://lutsai.ru/stm32/2016/03/08/Library-ssd1306-stm32.html
 *
 * Jihon Lee (Eziya): DS3231 Library for STM32 HAL
 * https://github.com/eziya/STM32_HAL_DS3231
 *
 * Tested with STM32CubeIDE v1.5.1 and FW_F4 V1.25.2
 */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "ssd1306.h"
#include "stm32_ds3231.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
I2C_HandleTypeDef hi2c1;

/* USER CODE BEGIN PV */
OledHandleTypedef oled = {
  .type = SSD1306_I2C,        // SH1106 or SSD1306
  .orientation = 1,           // 1: downward 0: upward
  .i2c = &hi2c1,              // I2C1 at RB9/RB8
  .i2c_address = 0x78         // 0x78 (0x3c<<1) or 0x7A (0x3D<<1)
};

_RTC rtc = {
    .Year = 21, .Month = 2, .Date = 18,
    .DaysOfWeek = THURSDAY,
    .Hour = 17, .Min = 0, .Sec = 0
};

uint8_t status;
char msg[80];
char dow[8][4] = {"Err", "SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};
char moy[13][4] = {"Err", "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL",
		           "AUG", "SEP", "OCT", "NOV", "DEC"};

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_I2C1_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_I2C1_Init();
  /* USER CODE BEGIN 2 */
  DS3231_Init(&hi2c1);
  status = DS3231_SetTime(&rtc);

  SSD1306_Init(&oled);        // Inicializálás
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1) {
	status = DS3231_GetTime(&rtc);
	sprintf(msg, "20%02d-%s-%02d %s",rtc.Year, moy[rtc.Month],rtc.Date, dow[rtc.DaysOfWeek]);
	SSD1306_GotoXY(5, 1);
	SSD1306_Puts(msg, &Font_7x10, SSD1306_COLOR_WHITE);
	sprintf(msg,"%02d:%02d:%02d",rtc.Hour,rtc.Min,rtc.Sec);
	SSD1306_GotoXY(12, 12);
	SSD1306_Puts(msg, &Font_11x18, SSD1306_COLOR_WHITE);
	SSD1306_UpdateScreen();     // Megjelenítés
	HAL_Delay(1000);
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_BYPASS;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 4;
  RCC_OscInitStruct.PLL.PLLN = 180;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 2;
  RCC_OscInitStruct.PLL.PLLR = 2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Activate the Over-Drive mode
  */
  if (HAL_PWREx_EnableOverDrive() != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 100000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
